///
/// \file packager.cc
/// \author K. Isom <kyle@imap.cc>
/// \date 2023-10-18
/// \brief Packaging tools for shimmering-clarity.
///
/// Copyright 2023 K. Isom <kyle@imap.cc>
///
/// Permission to use, copy, modify, and/or distribute this software for
/// any purpose with or without fee is hereby granted, provided that
/// the above copyright notice and this permission notice appear in all /// copies.
///
/// THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL
/// WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED
/// WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE
/// AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL
/// DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA
/// OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
/// TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
/// PERFORMANCE OF THIS SOFTWARE.
///

#include <filesystem>
#include <iostream>
#include <string>

#include <scsl/Commander.h>
#include <scsl/Flags.h>
#include <scsl/SimpleConfig.h>

#include "Debian.h"
#include "Generic.h"
#include "Utility.h"


void
usage(std::ostream &outs, int exc)
{
	outs << "sc3dev-packager: manage packages on gitea\n";
	outs << "usage:\n";
	outs << "\tsc3dev-packager [--component component] [--distribution distribution]\n";
	outs << "\t\t debian command packages ...\n";
	outs << "\tsc3dev-packager generic command packages ...\n";
	outs << "\nGlobal flags:\n";
	outs << "\t[--owner owner]\n";
	outs << "\t[--verbose]\n";
	outs << "\nCommands:\n";
	outs << "\t- delete: remove a package\n";
	outs << "\t- put: upload a package\n";

	exit(exc);
}


std::string
getDefaultConfigFile()
{
	auto *userEnv = getenv("USER");
	auto  user     = std::string(userEnv);

	if (user == "root") {
		return std::filesystem::path("/etc/sc3dev/packager.conf");
	}

	if (user.empty()) {
		return "";
	}

	userEnv       = getenv("HOME");
	auto userHome = std::filesystem::path(userEnv);

	userHome /= std::filesystem::path(".config");
	userHome /= std::filesystem::path("sc3dev");
	userHome /= std::filesystem::path("packager.conf");
	return userHome;
}


int
main(int argc, char *argv[])
{
	int retc = -1;
	scsl::Flags *flags = nullptr;
	std::string configFile = getDefaultConfigFile();
	std::string owner;
	std::string distribution;
	std::string component;
	bool verbose;

	flags = new scsl::Flags("sc3dev-packager", "package tooling for shimmering clarity");
	flags->Register("--config", configFile, "path to a configuration file");
	flags->Register("--component", defaultComponent, "Debian package component");
	flags->Register("--distribution", defaultDistribution, "Debian package distribution");
	flags->Register("--owner", defaultOwner, "package repository owner");
	flags->Register("--verbose", false, "use verbose CURL endpoints");

	auto status = flags->Parse(argc, argv);
	if (status != scsl::Flags::ParseStatus::OK) {
		std::cerr << "failed to parse flags: "
			  << scsl::Flags::ParseStatusToString(status)
			  << "\n";
		exit(1);
	}

	scsl::Commander commander;
	commander.Register(scsl::Subcommand("debian", 2, debianPackager));
	commander.Register(scsl::Subcommand("generic", 2, genericPackager));

	flags->GetString("--config", configFile);
	if (std::filesystem::exists(std::filesystem::path(configFile))) {
		if (scsl::SimpleConfig::LoadGlobal(configFile) != 0) {
			std::cerr << "[!] failed to read config file " << configFile << ".\n";
			exit(1);
		}
	}

	if (flags->GetString("--owner", owner)) {
		scsl::SimpleConfig::PutGlobal("owner", owner);
	}

	if (flags->GetString("--distribution", distribution)) {
		scsl::SimpleConfig::PutGlobal("distribution", distribution);
	}

	if (flags->GetString("--component", component)) {
		scsl::SimpleConfig::PutGlobal("component", component);
	}

	if (flags->GetBool("--verbose", verbose)) {
		scsl::SimpleConfig::PutGlobal("verbose", "true");
	}

	if (flags->NumArgs() < 1) {
		std::cerr << "[!] not enough arguments provided.\n";
		exit(1);
	}

	auto command = flags->Arg(0);
	auto args    = flags->Args();
	args.erase(args.begin());

	auto result = commander.Run(command, args);
	delete flags;

	switch (result) {
	case scsl::Subcommand::Status::OK:
		std::cout << "[+] OK\n";
		retc = 0;
		break;
	case scsl::Subcommand::Status::NotEnoughArgs:
		usage(std::cerr, 1);
		break;
	case scsl::Subcommand::Status::Failed:
		std::cerr << "[!] '" << command << "' failed\n";
		break;
	case scsl::Subcommand::Status::CommandNotRegistered:
		std::cerr << "[!] '" << command << "' not registered.\n";
		usage(std::cerr, 1);
		break;
	default:
		abort();
	}

	return retc;
}
